import java.util.*;

class Movie implements Comparable<Movie> {
  public String title;
  public int year;
  public double rating;

  /**
   * Constructs a Movie object.
   * 
   * @param title  - movie title.
   * @param year   - year the movie was released.
   * @param rating - IMDb rating (0 - 10)
   */
  public Movie(String title, int year, double rating) {
    this.title = title;
    this.year = year;
    this.rating = rating;
  }

  /**
   * Compares whether this movie rating was lower than some otherMovie.
   * 
   * @param otherMovie - the movie to compare with.
   * @return -1 if this movie rating is lower than the other movie,
   *         +1 if this movie rating is higher than the other movie,
   *         0 if the movie ratings are equal.
   */
  public int compareTo(Movie otherMovie) {
    if (rating < otherMovie.rating)
      return -1;
    else if (rating > otherMovie.rating)
      return 1;
    return 0;
  }

  /**
   * @return a String representation of the movie.
   */
  public String toString() {
    return title + " (" + year + "), rating = " + rating;
  }
}

/**
 * Helper class to compare movies by release year.
 */
class MovieYearComparator implements Comparator<Movie> {
  /**
   * Compares whether movie1 < movie2.
   * 
   * @param movie1 - left-hand-side movie in the comparison.
   * @param movie2 - right-hand-side movie in the comparison.
   * @return -1 if movie1 was released earlier.
   *         +1 if movie2 was released earlier.
   *         0 if both movies were released at the same time.
   */
  public int compare(Movie movie1, Movie movie2) {
    if (movie1.year < movie2.year)
      return -1;
    else if (movie1.year > movie2.year)
      return 1;
    return 0;
  }
}

/**
 * Helper class to compare movies by the length of their titles.
 */
class MovieTitleLengthComparator implements Comparator<Movie> {
  /**
   * Compares whether movie1 < movie2.
   * 
   * @param movie1 - left-hand-side movie in the comparison.
   * @param movie2 - right-hand-side movie in the comparison.
   * @return -1 if movie1 has a longer title.
   *         +1 if movie2 has a longer title.
   *         0 if both movies have the same number of characters in their titles.
   */
  public int compare(Movie movie1, Movie movie2) {
    if (movie1.title.length() < movie2.title.length())
      return -1;
    else if (movie1.title.length() > movie2.title.length())
      return 1;
    return 0;
  }
}

public class SortMovies {

  /**
   * Tests Arrays.sort using the Comparable<Movie> and a few Comparators.
   * 
   * @param args - unused.
   */
  public static void main(String[] args) {

    Movie[] movies = { new Movie("Toy Story", 1995, 8.3), new Movie("Coco", 2017, 8.4),
        new Movie("Monsters, Inc.", 2001, 8.1), new Movie("Brave", 2012, 7.1),
        new Movie("The Good Dinosaur", 2015, 6.7), new Movie("Inside Out", 2015, 8.1),
        new Movie("A Bug's Life", 1998, 7.2), new Movie("Luca", 2021, 7.4), new Movie("Finding Nemo", 2003, 8.2),
        new Movie("Turning Red", 2022, 7.0), new Movie("Ratatouille", 2007, 8.1),
        new Movie("The Incredibles", 2004, 8.0), new Movie("Cars", 2006, 7.2), new Movie("Onward", 2020, 7.4),
        new Movie("Elemental", 2023, 7.0), new Movie("WALL-E", 2008, 8.4), new Movie("Up", 2009, 8.3) };

    // Sort using the compareTo method (rating)
    Arrays.sort(movies);
    for (Movie m : movies) {
      System.out.println(m);
    }
    System.out.println();

    // Sort using one of the comparators
    MovieYearComparator cmp = new MovieYearComparator();
    //MovieTitleLengthComparator cmp = new MovieTitleLengthComparator();
    Arrays.sort(movies, cmp);
    for (Movie m : movies) {
      System.out.println(m);
    }
  }

}
